@@ -1,8 +1,8 @@
 # Copyright (C) 2016 JWCrypto Project Contributors - see LICENSE file
 
-import abc
 import os
 import struct
+from abc import ABCMeta, abstractmethod
 from binascii import hexlify, unhexlify
 
 from cryptography.exceptions import InvalidSignature
@@ -17,8 +17,6 @@ from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 from cryptography.hazmat.primitives.keywrap import aes_key_unwrap, aes_key_wrap
 from cryptography.hazmat.primitives.padding import PKCS7
 
-import six
-
 from jwcrypto.common import InvalidCEKeyLength
 from jwcrypto.common import InvalidJWAAlgorithm
 from jwcrypto.common import InvalidJWEKeyLength
@@ -31,33 +29,32 @@ from jwcrypto.jwk import JWK
 # Implements RFC 7518 - JSON Web Algorithms (JWA)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class JWAAlgorithm(object):
+class JWAAlgorithm(metaclass=ABCMeta):
 
-    @abc.abstractproperty
+    @property
+    @abstractmethod
     def name(self):
         """The algorithm Name"""
-        pass
 
-    @abc.abstractproperty
+    @property
+    @abstractmethod
     def description(self):
         """A short description"""
-        pass
 
-    @abc.abstractproperty
+    @property
+    @abstractmethod
     def keysize(self):
         """The actual/recommended/minimum key size"""
-        pass
 
-    @abc.abstractproperty
+    @property
+    @abstractmethod
     def algorithm_usage_location(self):
         """One of 'alg', 'enc' or 'JWK'"""
-        pass
 
-    @abc.abstractproperty
+    @property
+    @abstractmethod
     def algorithm_use(self):
         """One of 'sig', 'kex', 'enc'"""
-        pass
 
 
 def _bitsize(x):
@@ -161,7 +158,7 @@ class _RawNone(_RawJWS):
         return ''
 
     def verify(self, key, payload, signature):
-        if key.key_type != 'oct' or key.get_op_key() != '':
+        if key['kty'] != 'oct' or key.get_op_key() != '':
             raise InvalidSignature('The "none" signature cannot be verified')
 
 
@@ -353,8 +350,8 @@ class _RSA(_RawKeyMgmt):
     def _check_key(self, key):
         if not isinstance(key, JWK):
             raise ValueError('key is not a JWK object')
-        if key.key_type != 'RSA':
-            raise InvalidJWEKeyType('RSA', key.key_type)
+        if key['kty'] != 'RSA':
+            raise InvalidJWEKeyType('RSA', key['kty'])
 
     # FIXME: get key size and insure > 2048 bits
     def wrap(self, key, bitsize, cek, headers):
@@ -441,8 +438,8 @@ class _AesKw(_RawKeyMgmt):
     def _get_key(self, key, op):
         if not isinstance(key, JWK):
             raise ValueError('key is not a JWK object')
-        if key.key_type != 'oct':
-            raise InvalidJWEKeyType('oct', key.key_type)
+        if key['kty'] != 'oct':
+            raise InvalidJWEKeyType('oct', key['kty'])
         rk = base64url_decode(key.get_op_key(op))
         if _bitsize(rk) != self.keysize:
             raise InvalidJWEKeyLength(self.keysize, _bitsize(rk))
@@ -503,8 +500,8 @@ class _AesGcmKw(_RawKeyMgmt):
     def _get_key(self, key, op):
         if not isinstance(key, JWK):
             raise ValueError('key is not a JWK object')
-        if key.key_type != 'oct':
-            raise InvalidJWEKeyType('oct', key.key_type)
+        if key['kty'] != 'oct':
+            raise InvalidJWEKeyType('oct', key['kty'])
         rk = base64url_decode(key.get_op_key(op))
         if _bitsize(rk) != self.keysize:
             raise InvalidJWEKeyLength(self.keysize, _bitsize(rk))
@@ -583,10 +580,15 @@ class _Pbes2HsAesKw(_RawKeyMgmt):
         self.aeskwmap = {128: _A128KW, 192: _A192KW, 256: _A256KW}
 
     def _get_key(self, alg, key, p2s, p2c):
-        if isinstance(key, bytes):
-            plain = key
+        if not isinstance(key, JWK):
+            # backwards compatiblity for old interface
+            if isinstance(key, bytes):
+                plain = key
+            else:
+                plain = key.encode('utf8')
         else:
-            plain = key.encode('utf8')
+            plain = base64url_decode(key.get_op_key())
+
         salt = bytes(self.name.encode('utf8')) + b'\x00' + p2s
 
         if self.hashsize == 256:
@@ -669,8 +671,8 @@ class _Direct(_RawKeyMgmt, JWAAlgorithm):
     def _check_key(self, key):
         if not isinstance(key, JWK):
             raise ValueError('key is not a JWK object')
-        if key.key_type != 'oct':
-            raise InvalidJWEKeyType('oct', key.key_type)
+        if key['kty'] != 'oct':
+            raise InvalidJWEKeyType('oct', key['kty'])
 
     def wrap(self, key, bitsize, cek, headers):
         self._check_key(key)
@@ -706,12 +708,12 @@ class _EcdhEs(_RawKeyMgmt, JWAAlgorithm):
     def _check_key(self, key):
         if not isinstance(key, JWK):
             raise ValueError('key is not a JWK object')
-        if key.key_type not in ['EC', 'OKP']:
-            raise InvalidJWEKeyType('EC or OKP', key.key_type)
-        if key.key_type == 'OKP':
-            if key.key_curve not in ['X25519', 'X448']:
+        if key['kty'] not in ['EC', 'OKP']:
+            raise InvalidJWEKeyType('EC or OKP', key['kty'])
+        if key['kty'] == 'OKP':
+            if key['crv'] not in ['X25519', 'X448']:
                 raise InvalidJWEKeyType('X25519 or X448',
-                                        key.key_curve)
+                                        key['crv'])
 
     def _derive(self, privkey, pubkey, alg, bitsize, headers):
         # OtherInfo is defined in NIST SP 56A 5.8.1.2.1
@@ -759,7 +761,7 @@ class _EcdhEs(_RawKeyMgmt, JWAAlgorithm):
         else:
             alg = headers['alg']
 
-        epk = JWK.generate(kty=key.key_type, crv=key.key_curve)
+        epk = JWK.generate(kty=key['kty'], crv=key['crv'])
         dk = self._derive(epk.get_op_key('unwrapKey'),
                           key.get_op_key('wrapKey'),
                           alg, dk_size, headers)
@@ -835,13 +837,13 @@ class _EdDsa(_RawJWS, JWAAlgorithm):
 
     def sign(self, key, payload):
 
-        if key.key_curve in ['Ed25519', 'Ed448']:
+        if key['crv'] in ['Ed25519', 'Ed448']:
             skey = key.get_op_key('sign')
             return skey.sign(payload)
         raise NotImplementedError
 
     def verify(self, key, payload, signature):
-        if key.key_curve in ['Ed25519', 'Ed448']:
+        if key['crv'] in ['Ed25519', 'Ed448']:
             pkey = key.get_op_key('verify')
             return pkey.verify(signature, payload)
         raise NotImplementedError
@@ -1099,21 +1101,21 @@ class JWA(object):
         try:
             return cls.instantiate_alg(name, use='sig')
         except KeyError:
-            raise InvalidJWAAlgorithm(
-                '%s is not a valid Signign algorithm name' % name)
+            raise InvalidJWAAlgorithm('%s is not a valid Signign algorithm'
+                                      ' name' % name) from None
 
     @classmethod
     def keymgmt_alg(cls, name):
         try:
             return cls.instantiate_alg(name, use='kex')
         except KeyError:
-            raise InvalidJWAAlgorithm(
-                '%s is not a valid Key Management algorithm name' % name)
+            raise InvalidJWAAlgorithm('%s is not a valid Key Management'
+                                      ' algorithm name' % name) from None
 
     @classmethod
     def encryption_alg(cls, name):
         try:
             return cls.instantiate_alg(name, use='enc')
         except KeyError:
-            raise InvalidJWAAlgorithm(
-                '%s is not a valid Encryption algorithm name' % name)
+            raise InvalidJWAAlgorithm('%s is not a valid Encryption'
+                                      ' algorithm name' % name) from None
