@@ -11,7 +11,7 @@ from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric import rsa
 
-from six import iteritems
+from deprecated import deprecated
 
 from jwcrypto.common import JWException
 from jwcrypto.common import base64url_decode, base64url_encode
@@ -151,7 +151,7 @@ JWKParamsRegistry = {
 """Registry of valid key parameters"""
 
 # RFC 7518 - 7.6 , RFC 8037 - 5
-# secp256k1 - https://tools.ietf.org/html/draft-ietf-cose-webauthn-algorithms
+# RFC 8812 - 4.4
 JWKEllipticCurveRegistry = {'P-256': 'P-256 curve',
                             'P-384': 'P-384 curve',
                             'P-521': 'P-521 curve',
@@ -261,10 +261,8 @@ class InvalidJWKValue(JWException):
     on the key type or other constraints.
     """
 
-    pass
 
-
-class JWK(object):
+class JWK(dict):
     """JSON Web Key object
 
     This object represent a Key.
@@ -302,9 +300,7 @@ class JWK(object):
         :raises InvalidJWKValue: if incorrect or inconsistent parameters
             are provided.
         """
-        self._params = dict()
-        self._key = dict()
-        self._unknown = dict()
+        super(JWK, self).__init__()
 
         if 'generate' in kwargs:
             self.generate_key(**kwargs)
@@ -318,8 +314,8 @@ class JWK(object):
         try:
             kty = kwargs['kty']
             gen = getattr(obj, '_generate_%s' % kty)
-        except (KeyError, AttributeError):
-            raise InvalidJWKType(kty)
+        except (KeyError, AttributeError) as e:
+            raise InvalidJWKType(kty) from e
         gen(kwargs)
         return obj
 
@@ -328,8 +324,8 @@ class JWK(object):
         try:
             kty = params.pop('generate')
             gen = getattr(self, '_generate_%s' % kty)
-        except (KeyError, AttributeError):
-            raise InvalidJWKType(kty)
+        except (KeyError, AttributeError) as e:
+            raise InvalidJWKType(kty) from e
 
         gen(params)
 
@@ -341,8 +337,8 @@ class JWK(object):
             try:
                 from jwcrypto.jwa import JWA
                 alg = JWA.instantiate_alg(params['alg'])
-            except KeyError:
-                raise ValueError("Invalid 'alg' parameter")
+            except KeyError as e:
+                raise ValueError("Invalid 'alg' parameter") from e
             size = alg.keysize
         return size
 
@@ -437,11 +433,12 @@ class JWK(object):
 
     def _import_pyca_pub_ec(self, key, **params):
         pn = key.public_numbers()
+        key_size = pn.curve.key_size
         params.update(
             kty='EC',
             crv=JWKpycaCurveMap[key.curve.name],
-            x=self._encode_int(pn.x),
-            y=self._encode_int(pn.y),
+            x=self._encode_int(pn.x, key_size),
+            y=self._encode_int(pn.y, key_size),
         )
         self.import_key(**params)
 
@@ -450,13 +447,13 @@ class JWK(object):
             raise InvalidJWKValue('Must specify "crv" for OKP key generation')
         try:
             key = _OKP_CURVES_TABLE[params['crv']].privkey.generate()
-        except KeyError:
+        except KeyError as e:
             raise InvalidJWKValue('"%s" is not a supported curve for the '
-                                  'OKP key type' % params['crv'])
+                                  'OKP key type' % params['crv']) from e
         self._import_pyca_pri_okp(key, **params)
 
     def _okp_curve_from_pyca_key(self, key):
-        for name, val in iteritems(_OKP_CURVES_TABLE):
+        for name, val in _OKP_CURVES_TABLE.items():
             if isinstance(key, (val.pubkey, val.privkey)):
                 return name
         raise InvalidJWKValue('Invalid OKP Key object %r' % key)
@@ -486,82 +483,90 @@ class JWK(object):
         self.import_key(**params)
 
     def import_key(self, **kwargs):
+        newkey = dict()
+        key_vals = 0
+
         names = list(kwargs.keys())
 
         for name in list(JWKParamsRegistry.keys()):
             if name in kwargs:
-                self._params[name] = kwargs[name]
+                newkey[name] = kwargs[name]
                 while name in names:
                     names.remove(name)
 
-        kty = self._params.get('kty', None)
+        kty = newkey.get('kty')
         if kty not in JWKTypesRegistry:
             raise InvalidJWKType(kty)
 
         for name in list(JWKValuesRegistry[kty].keys()):
             if name in kwargs:
-                self._key[name] = kwargs[name]
+                newkey[name] = kwargs[name]
+                key_vals += 1
                 while name in names:
                     names.remove(name)
 
-        for name, val in iteritems(JWKValuesRegistry[kty]):
-            if val.required and name not in self._key:
+        for name, val in JWKValuesRegistry[kty].items():
+            if val.required and name not in newkey:
                 raise InvalidJWKValue('Missing required value %s' % name)
-            if val.type == ParmType.unsupported and name in self._key:
+            if val.type == ParmType.unsupported and name in newkey:
                 raise InvalidJWKValue('Unsupported parameter %s' % name)
-            if val.type == ParmType.b64 and name in self._key:
+            if val.type == ParmType.b64 and name in newkey:
                 # Check that the value is base64url encoded
                 try:
-                    base64url_decode(self._key[name])
-                except Exception:  # pylint: disable=broad-except
+                    base64url_decode(newkey[name])
+                except Exception as e:  # pylint: disable=broad-except
                     raise InvalidJWKValue(
                         '"%s" is not base64url encoded' % name
-                    )
-            if val[3] == ParmType.b64u and name in self._key:
+                    ) from e
+            if val.type == ParmType.b64u and name in newkey:
                 # Check that the value is Base64urlUInt encoded
                 try:
-                    self._decode_int(self._key[name])
-                except Exception:  # pylint: disable=broad-except
+                    self._decode_int(newkey[name])
+                except Exception as e:  # pylint: disable=broad-except
                     raise InvalidJWKValue(
                         '"%s" is not Base64urlUInt encoded' % name
-                    )
+                    ) from e
 
         # Unknown key parameters are allowed
-        # Let's just store them out of the way
         for name in names:
-            self._unknown[name] = kwargs[name]
+            newkey[name] = kwargs[name]
 
-        if len(self._key) == 0:
+        if key_vals == 0:
             raise InvalidJWKValue('No Key Values found')
 
         # check key_ops
-        if 'key_ops' in self._params:
-            for ko in self._params['key_ops']:
+        if 'key_ops' in newkey:
+            for ko in newkey['key_ops']:
                 c = 0
-                for cko in self._params['key_ops']:
+                for cko in newkey['key_ops']:
                     if ko == cko:
                         c += 1
                 if c != 1:
                     raise InvalidJWKValue('Duplicate values in "key_ops"')
 
         # check use/key_ops consistency
-        if 'use' in self._params and 'key_ops' in self._params:
+        if 'use' in newkey and 'key_ops' in newkey:
             sigl = ['sign', 'verify']
             encl = ['encrypt', 'decrypt', 'wrapKey', 'unwrapKey',
                     'deriveKey', 'deriveBits']
-            if self._params['use'] == 'sig':
+            if newkey['use'] == 'sig':
                 for op in encl:
-                    if op in self._params['key_ops']:
+                    if op in newkey['key_ops']:
                         raise InvalidJWKValue('Incompatible "use" and'
                                               ' "key_ops" values specified at'
                                               ' the same time')
-            elif self._params['use'] == 'enc':
+            elif newkey['use'] == 'enc':
                 for op in sigl:
-                    if op in self._params['key_ops']:
+                    if op in newkey['key_ops']:
                         raise InvalidJWKValue('Incompatible "use" and'
                                               ' "key_ops" values specified at'
                                               ' the same time')
 
+        self.clear()
+        # must set 'kty' as first item
+        self.__setitem__('kty', newkey['kty'])
+        self.update(newkey)
+
     @classmethod
     def from_json(cls, key):
         """Creates a RFC 7517 JWK from the standard JSON format.
@@ -572,7 +577,7 @@ class JWK(object):
         try:
             jkey = json_decode(key)
         except Exception as e:  # pylint: disable=broad-except
-            raise InvalidJWKValue(e)
+            raise InvalidJWKValue from e
         obj.import_key(**jkey)
         return obj
 
@@ -607,22 +612,20 @@ class JWK(object):
         if not self.has_public:
             raise InvalidJWKType("No public key available")
         pub = {}
-        preg = JWKParamsRegistry
-        for name in preg:
-            if preg[name].public:
-                if name in self._params:
-                    pub[name] = self._params[name]
-        reg = JWKValuesRegistry[self._params['kty']]
-        for param in reg:
-            if reg[param].public:
-                pub[param] = self._key[param]
+        reg = JWKParamsRegistry
+        for name in reg:
+            if reg[name].public:
+                if name in self.keys():
+                    pub[name] = self.get(name)
+        reg = JWKValuesRegistry[self.get('kty')]
+        for name in reg:
+            if reg[name].public:
+                pub[name] = self.get(name)
         return pub
 
     def _export_all(self, as_dict=False):
         d = dict()
-        d.update(self._params)
-        d.update(self._key)
-        d.update(self._unknown)
+        d.update(self)
         if as_dict is True:
             return d
         return json_encode(d)
@@ -648,48 +651,52 @@ class JWK(object):
 
     @property
     def has_public(self):
-        """Whether this JWK has an asymmetric Public key."""
+        """Whether this JWK has an asymmetric Public key value."""
         if self.is_symmetric:
             return False
-        reg = JWKValuesRegistry[self._params['kty']]
-        for value in reg:
-            if reg[value].public and value in self._key:
+        reg = JWKValuesRegistry[self.get('kty')]
+        for name in reg:
+            if reg[name].public and name in self.keys():
                 return True
+        return False
 
     @property
     def has_private(self):
-        """Whether this JWK has an asymmetric key Private key."""
+        """Whether this JWK has an asymmetric Private key value."""
         if self.is_symmetric:
             return False
-        reg = JWKValuesRegistry[self._params['kty']]
-        for value in reg:
-            if not reg[value].public and value in self._key:
+        reg = JWKValuesRegistry[self.get('kty')]
+        for name in reg:
+            if not reg[name].public and name in self.keys():
                 return True
         return False
 
     @property
     def is_symmetric(self):
         """Whether this JWK is a symmetric key."""
-        return self.key_type == 'oct'
+        return self.get('kty') == 'oct'
 
     @property
+    @deprecated
     def key_type(self):
         """The Key type"""
-        return self._params.get('kty', None)
+        return self.get('kty')
 
     @property
+    @deprecated
     def key_id(self):
         """The Key ID.
         Provided by the kid parameter if present, otherwise returns None.
         """
-        return self._params.get('kid', None)
+        return self.get('kid')
 
     @property
+    @deprecated
     def key_curve(self):
         """The Curve Name."""
-        if self._params['kty'] not in ['EC', 'OKP']:
+        if self.get('kty') not in ['EC', 'OKP']:
             raise InvalidJWKType('Not an EC or OKP key')
-        return self._key['crv']
+        return self.get('crv')
 
     def get_curve(self, arg):
         """Gets the Elliptic Curve associated with the key.
@@ -699,20 +706,20 @@ class JWK(object):
         :raises InvalidJWKType: the key is not an EC or OKP key.
         :raises InvalidJWKValue: if the curve names is invalid.
         """
-        k = self._key
-        if self._params['kty'] not in ['EC', 'OKP']:
+        crv = self.get('crv')
+        if self.get('kty') not in ['EC', 'OKP']:
             raise InvalidJWKType('Not an EC or OKP key')
-        if arg and k['crv'] != arg:
+        if arg and crv != arg:
             raise InvalidJWKValue('Curve requested is "%s", but '
-                                  'key curve is "%s"' % (arg, k['crv']))
+                                  'key curve is "%s"' % (arg, crv))
 
-        return self._get_curve_by_name(k['crv'])
+        return self._get_curve_by_name(crv)
 
     def _check_constraints(self, usage, operation):
-        use = self._params.get('use', None)
+        use = self.get('use')
         if use and use != usage:
             raise InvalidJWKUsage(usage, use)
-        ops = self._params.get('key_ops', None)
+        ops = self.get('key_ops')
         if ops:
             if not isinstance(ops, list):
                 ops = [ops]
@@ -723,65 +730,72 @@ class JWK(object):
     def _decode_int(self, n):
         return int(hexlify(base64url_decode(n)), 16)
 
-    def _rsa_pub(self, k):
-        return rsa.RSAPublicNumbers(self._decode_int(k['e']),
-                                    self._decode_int(k['n']))
+    def _rsa_pub(self):
+        e = self._decode_int(self.get('e'))
+        n = self._decode_int(self.get('n'))
+        return rsa.RSAPublicNumbers(e, n)
 
-    def _rsa_pri(self, k):
-        return rsa.RSAPrivateNumbers(self._decode_int(k['p']),
-                                     self._decode_int(k['q']),
-                                     self._decode_int(k['d']),
-                                     self._decode_int(k['dp']),
-                                     self._decode_int(k['dq']),
-                                     self._decode_int(k['qi']),
-                                     self._rsa_pub(k))
+    def _rsa_pri(self):
+        p = self._decode_int(self.get('p'))
+        q = self._decode_int(self.get('q'))
+        d = self._decode_int(self.get('d'))
+        dp = self._decode_int(self.get('dp'))
+        dq = self._decode_int(self.get('dq'))
+        qi = self._decode_int(self.get('qi'))
+        return rsa.RSAPrivateNumbers(p, q, d, dp, dq, qi, self._rsa_pub())
 
-    def _ec_pub(self, k, curve):
-        return ec.EllipticCurvePublicNumbers(self._decode_int(k['x']),
-                                             self._decode_int(k['y']),
-                                             self.get_curve(curve))
+    def _ec_pub(self, curve):
+        x = self._decode_int(self.get('x'))
+        y = self._decode_int(self.get('y'))
+        return ec.EllipticCurvePublicNumbers(x, y, self.get_curve(curve))
 
-    def _ec_pri(self, k, curve):
-        return ec.EllipticCurvePrivateNumbers(self._decode_int(k['d']),
-                                              self._ec_pub(k, curve))
+    def _ec_pri(self, curve):
+        d = self._decode_int(self.get('d'))
+        return ec.EllipticCurvePrivateNumbers(d, self._ec_pub(curve))
 
-    def _okp_pub(self, k):
+    def _okp_pub(self):
+        crv = self.get('crv')
         try:
-            pubkey = _OKP_CURVES_TABLE[k['crv']].pubkey
-        except KeyError:
-            raise InvalidJWKValue('Unknown curve "%s"' % k['crv'])
+            pubkey = _OKP_CURVES_TABLE[crv].pubkey
+        except KeyError as e:
+            raise InvalidJWKValue('Unknown curve "%s"' % crv) from e
 
-        return pubkey.from_public_bytes(base64url_decode(k['x']))
+        x = base64url_decode(self.get('x'))
+        return pubkey.from_public_bytes(x)
 
-    def _okp_pri(self, k):
+    def _okp_pri(self):
+        crv = self.get('crv')
         try:
-            privkey = _OKP_CURVES_TABLE[k['crv']].privkey
-        except KeyError:
-            raise InvalidJWKValue('Unknown curve "%s"' % k['crv'])
+            privkey = _OKP_CURVES_TABLE[crv].privkey
+        except KeyError as e:
+            raise InvalidJWKValue('Unknown curve "%s"' % crv) from e
 
-        return privkey.from_private_bytes(base64url_decode(k['d']))
+        d = base64url_decode(self.get('d'))
+        return privkey.from_private_bytes(d)
 
     def _get_public_key(self, arg=None):
-        if self._params['kty'] == 'oct':
-            return self._key['k']
-        elif self._params['kty'] == 'RSA':
-            return self._rsa_pub(self._key).public_key(default_backend())
-        elif self._params['kty'] == 'EC':
-            return self._ec_pub(self._key, arg).public_key(default_backend())
-        elif self._params['kty'] == 'OKP':
-            return self._okp_pub(self._key)
+        ktype = self.get('kty')
+        if ktype == 'oct':
+            return self.get('k')
+        elif ktype == 'RSA':
+            return self._rsa_pub().public_key(default_backend())
+        elif ktype == 'EC':
+            return self._ec_pub(arg).public_key(default_backend())
+        elif ktype == 'OKP':
+            return self._okp_pub()
         else:
             raise NotImplementedError
 
     def _get_private_key(self, arg=None):
-        if self._params['kty'] == 'oct':
-            return self._key['k']
-        elif self._params['kty'] == 'RSA':
-            return self._rsa_pri(self._key).private_key(default_backend())
-        elif self._params['kty'] == 'EC':
-            return self._ec_pri(self._key, arg).private_key(default_backend())
-        elif self._params['kty'] == 'OKP':
-            return self._okp_pri(self._key)
+        ktype = self.get('kty')
+        if ktype == 'oct':
+            return self.get('k')
+        elif ktype == 'RSA':
+            return self._rsa_pri().private_key(default_backend())
+        elif ktype == 'EC':
+            return self._ec_pri(arg).private_key(default_backend())
+        elif ktype == 'OKP':
+            return self._okp_pri()
         else:
             raise NotImplementedError
 
@@ -801,13 +815,13 @@ class JWK(object):
         :raises InvalidJWKUsage: if the use constraints do not permit
          the operation.
         """
-        validops = self._params.get('key_ops',
-                                    list(JWKOperationsRegistry.keys()))
+        validops = self.get('key_ops',
+                            list(JWKOperationsRegistry.keys()))
         if validops is not list:
             validops = [validops]
         if operation is None:
-            if self._params['kty'] == 'oct':
-                return self._key['k']
+            if self.get('kty') == 'oct':
+                return self.get('k')
             raise InvalidJWKOperation(operation, validops)
         elif operation == 'sign':
             self._check_constraints('sig', operation)
@@ -840,7 +854,7 @@ class JWK(object):
         else:
             raise InvalidJWKValue('Unknown key object %r' % key)
 
-    def import_from_pem(self, data, password=None):
+    def import_from_pem(self, data, password=None, kid=None):
         """Imports a key from data loaded from a PEM file.
         The key may be encrypted with a password.
         Private keys (PKCS#8 format), public keys, and X509 certificate's
@@ -865,10 +879,13 @@ class JWK(object):
                         data, backend=default_backend())
                     key = cert.public_key()
                 except ValueError:
+                    # pylint: disable=raise-missing-from
                     raise e
 
         self.import_from_pyca(key)
-        self._params['kid'] = self.thumbprint()
+        if kid is None:
+            kid = self.thumbprint()
+        self.__setitem__('kid', kid)
 
     def export_to_pem(self, private_key=False, password=False):
         """Exports keys to a data buffer suitable to be stored as a PEM file.
@@ -929,14 +946,147 @@ class JWK(object):
         :param hashalg: A hash function (defaults to SHA256)
         """
 
-        t = {'kty': self._params['kty']}
-        for name, val in iteritems(JWKValuesRegistry[t['kty']]):
+        t = {'kty': self.get('kty')}
+        for name, val in JWKValuesRegistry[t['kty']].items():
             if val.required:
-                t[name] = self._key[name]
+                t[name] = self.get(name)
         digest = hashes.Hash(hashalg, backend=default_backend())
         digest.update(bytes(json_encode(t).encode('utf8')))
         return base64url_encode(digest.finalize())
 
+    # Methods to constrain what this dict allows
+    def __setitem__(self, item, value):
+        kty = self.get('kty')
+
+        if item == 'kty':
+            if kty is None:
+                if value not in JWKTypesRegistry:
+                    raise InvalidJWKType(value)
+                super(JWK, self).__setitem__(item, value)
+                return
+            elif kty != value:
+                raise ValueError('Cannot change key type')
+
+        # Check if item is a key value and verify its format
+        if item in list(JWKValuesRegistry[kty].keys()):
+            if JWKValuesRegistry[kty][item].type == ParmType.b64:
+                try:
+                    v = base64url_decode(value)
+                    # empty values are also invalid except for the
+                    # special case of 'oct' key where an empty value
+                    # is used to indicate a 'None' key
+                    if v == b'' and kty != 'oct' and item != 'k':
+                        raise ValueError
+                except Exception as e:  # pylint: disable=broad-except
+                    raise InvalidJWKValue(
+                        '"%s" is not base64url encoded' % item
+                    ) from e
+            elif JWKValuesRegistry[kty][item].type == ParmType.b64u:
+                try:
+                    self._decode_int(value)
+                except Exception as e:  # pylint: disable=broad-except
+                    raise InvalidJWKValue(
+                        '"%s" is not Base64urlUInt encoded' % item
+                    ) from e
+            super(JWK, self).__setitem__(item, value)
+            return
+
+        # If not a key param check if it is a know parameter
+        if item in list(JWKParamsRegistry.keys()):
+            super(JWK, self).__setitem__(item, value)
+            return
+
+        # if neither a key param nor a known parameter, check if we are
+        # trying to set a parameter for a different key type and refuse
+        # in this case.
+        for name in list(JWKTypesRegistry.keys()):
+            if name == kty:
+                continue
+            if item in list(JWKValuesRegistry[name].keys()):
+                raise KeyError("Cannot set '{}' on '{}' key type".format(
+                               item, kty))
+
+        # ok if we've come this far it means we have an unknown parameter
+        super(JWK, self).__setitem__(item, value)
+
+    def update(self, *args, **kwargs):
+        for k, v in dict(*args, **kwargs).items():
+            self.__setitem__(k, v)
+
+    def setdefault(self, key, default=None):
+        if key not in self.keys():
+            self.__setitem__(key, default)
+        return self.get(key)
+
+    def __delitem__(self, item):
+        param = self.get(item)
+        if param is None:
+            raise KeyError(item)
+
+        if item == 'kty':
+            for name in list(JWKValuesRegistry[param].keys()):
+                if self.get(name) is not None:
+                    raise KeyError("Cannot remove 'kty', values present")
+
+        super(JWK, self).__delitem__(item)
+
+    def __eq__(self, other):
+        if not isinstance(other, JWK):
+            return NotImplemented
+
+        return self.thumbprint() == other.thumbprint() and \
+            self.get('kid') == other.get('kid')
+
+    def __hash__(self):
+        return hash((self.thumbprint(), self.get('kid')))
+
+    def __getattr__(self, item):
+        try:
+            if item in JWKParamsRegistry.keys():
+                if item in self.keys():
+                    return self.get(item)
+            kty = self.get('kty')
+            if kty is not None:
+                if item in list(JWKValuesRegistry[kty].keys()):
+                    if item in self.keys():
+                        return self.get(item)
+            raise KeyError
+        except KeyError:
+            raise AttributeError(item) from None
+
+    def __setattr__(self, item, value):
+        try:
+            if item in JWKParamsRegistry.keys():
+                self.__setitem__(item, value)
+            for name in list(JWKTypesRegistry.keys()):
+                if item in list(JWKValuesRegistry[name].keys()):
+                    self.__setitem__(item, value)
+            super(JWK, self).__setattr__(item, value)
+        except KeyError:
+            raise AttributeError(item) from None
+
+    @classmethod
+    def from_password(cls, password):
+        """Creates a symmetric JWK key from a user password.
+
+        :param key: A password in utf8 format.
+        """
+        obj = cls()
+        params = {'kty': 'oct'}
+        try:
+            params['k'] = base64url_encode(password.encode('utf8'))
+        except Exception as e:  # pylint: disable=broad-except
+            raise InvalidJWKValue from e
+        obj.import_key(**params)
+        return obj
+
+    # Prevent accidental disclosure of key material via repr()
+    def __repr__(self):
+        repr_dict = dict()
+        repr_dict['kid'] = self.get('kid', 'Missing Key ID')
+        repr_dict['thumbprint'] = self.thumbprint()
+        return json_encode(repr_dict)
+
 
 class _JWKkeys(set):
 
@@ -971,15 +1121,20 @@ class JWKSet(dict):
         return self['keys'].__contains__(key)
 
     def __setitem__(self, key, val):
-        if key == 'keys':
+        if key == 'keys' and not isinstance(val, _JWKkeys):
             self['keys'].add(val)
         else:
             super(JWKSet, self).__setitem__(key, val)
 
     def update(self, *args, **kwargs):
-        for k, v in iteritems(dict(*args, **kwargs)):
+        for k, v in dict(*args, **kwargs).items():
             self.__setitem__(k, v)
 
+    def setdefault(self, key, default=None):
+        if key not in self.keys():
+            self.__setitem__(key, default)
+        return self.get(key)
+
     def add(self, elem):
         self['keys'].add(elem)
 
@@ -993,7 +1148,7 @@ class JWKSet(dict):
                               a JSON object
         """
         exp_dict = dict()
-        for k, v in iteritems(self):
+        for k, v in self.items():
             if k == 'keys':
                 keys = list()
                 for jwk in v:
@@ -1011,13 +1166,13 @@ class JWKSet(dict):
         """
         try:
             jwkset = json_decode(keyset)
-        except Exception:  # pylint: disable=broad-except
-            raise InvalidJWKValue()
+        except Exception as e:  # pylint: disable=broad-except
+            raise InvalidJWKValue from e
 
         if 'keys' not in jwkset:
-            raise InvalidJWKValue()
+            raise InvalidJWKValue
 
-        for k, v in iteritems(jwkset):
+        for k, v in jwkset.items():
             if k == 'keys':
                 for jwk in v:
                     self['keys'].add(JWK(**jwk))
@@ -1039,6 +1194,17 @@ class JWKSet(dict):
         :param kid: the 'kid' key identifier.
         """
         for jwk in self['keys']:
-            if jwk.key_id == kid:
+            if jwk.get('kid') == kid:
                 return jwk
         return None
+
+    def __repr__(self):
+        repr_dict = dict()
+        for k, v in self.items():
+            if k == 'keys':
+                keys = list()
+                for jwk in v:
+                    keys.append(repr(jwk))
+                v = keys
+            repr_dict[k] = v
+        return json_encode(repr_dict)
