@@ -3,8 +3,6 @@
 import time
 import uuid
 
-from six import string_types
-
 from jwcrypto.common import JWException, json_decode, json_encode
 from jwcrypto.jwe import JWE
 from jwcrypto.jwk import JWK, JWKSet
@@ -199,7 +197,7 @@ class JWT(object):
         if check_claims is not None:
             self._check_claims = check_claims
 
-        if claims:
+        if claims is not None:
             self.claims = claims
 
         if jwt is not None:
@@ -306,17 +304,17 @@ class JWT(object):
     def _check_string_claim(self, name, claims):
         if name not in claims:
             return
-        if not isinstance(claims[name], string_types):
+        if not isinstance(claims[name], str):
             raise JWTInvalidClaimFormat("Claim %s is not a StringOrURI type")
 
     def _check_array_or_string_claim(self, name, claims):
         if name not in claims:
             return
         if isinstance(claims[name], list):
-            if any(not isinstance(claim, string_types) for claim in claims):
+            if any(not isinstance(claim, str) for claim in claims):
                 raise JWTInvalidClaimFormat(
                     "Claim %s contains non StringOrURI types" % (name, ))
-        elif not isinstance(claims[name], string_types):
+        elif not isinstance(claims[name], str):
             raise JWTInvalidClaimFormat(
                 "Claim %s is not a StringOrURI type" % (name, ))
 
@@ -325,9 +323,9 @@ class JWT(object):
             return
         try:
             int(claims[name])
-        except ValueError:
+        except ValueError as e:
             raise JWTInvalidClaimFormat(
-                "Claim %s is not an integer" % (name, ))
+                "Claim %s is not an integer" % (name, )) from e
 
     def _check_exp(self, claim, limit, leeway):
         if claim < limit - leeway:
@@ -347,6 +345,7 @@ class JWT(object):
         self._check_integer_claim('nbf', claims)
         self._check_integer_claim('iat', claims)
         self._check_string_claim('jti', claims)
+        self._check_string_claim('typ', claims)
 
         if self._check_claims is None:
             if 'exp' in claims:
@@ -363,10 +362,11 @@ class JWT(object):
             claims = json_decode(self.claims)
             if not isinstance(claims, dict):
                 raise ValueError()
-        except ValueError:
+        except ValueError as e:
             if self._check_claims is not None:
-                raise JWTInvalidClaimFormat(
-                    "Claims check requested but claims is not a json dict")
+                raise JWTInvalidClaimFormat("Claims check requested "
+                                            "but claims is not a json "
+                                            "dict") from e
             return
 
         self._check_default_claims(claims)
@@ -407,12 +407,28 @@ class JWT(object):
                 else:
                     self._check_nbf(claims[name], time.time(), self._leeway)
 
+            elif name == 'typ':
+                if value is not None:
+                    if self.norm_typ(value) != self.norm_typ(claims[name]):
+                        raise JWTInvalidClaimValue("Invalid '%s' value. '%s'"
+                                                   " does not normalize to "
+                                                   "'%s'" % (name,
+                                                             claims[name],
+                                                             value))
+
             else:
                 if value is not None and value != claims[name]:
                     raise JWTInvalidClaimValue(
                         "Invalid '%s' value. Expected '%s' got '%s'" % (
                             name, value, claims[name]))
 
+    def norm_typ(self, val):
+        lc = val.lower()
+        if '/' in lc:
+            return lc
+        else:
+            return 'application/' + lc
+
     def make_signed_token(self, key):
         """Signs the payload.
 
@@ -440,6 +456,8 @@ class JWT(object):
         """
 
         t = JWE(self.claims, self.header)
+        if self._algs:
+            t.allowed_algs = self._algs
         t.add_recipient(key)
         self.token = t
 
@@ -489,7 +507,7 @@ class JWT(object):
                         self.deserializelog.append("Success")
                         break
                     except Exception as e:  # pylint: disable=broad-except
-                        keyid = k.key_id
+                        keyid = k.get('kid')
                         if keyid is None:
                             keyid = k.thumbprint()
                         self.deserializelog.append('Key [%s] failed: [%s]' % (
